package com.hs.admin.service.impl;

import com.hs.admin.bean.SysTask;
import com.hs.admin.service.JobManagerService;
import com.hs.admin.util.JsonUtils;
import org.quartz.*;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.net.URL;

/**
 * @author xieheng
 */
@Service
public class JobManagerServiceImpl implements JobManagerService {
    @Resource
    Scheduler scheduler;

    /**
     * 添加任务
     *
     * @return
     */
    @SuppressWarnings("unchecked")
    @Override
    public boolean addJob(SysTask task) {
        try {
            // 创建jobDetail实例，绑定Job实现类
            // 指明job的名称，所在组的名称，以及绑定job类
            Class<? extends Job> jobClass = (Class<? extends Job>) (Class.forName(task.getBeanClass()).newInstance().getClass());
            // 任务名称和组构成任务key
            JobDetail jobDetail = JobBuilder.newJob(jobClass).withIdentity(task.getJobName(), task.getJobGroup()).build();
            // 定义调度触发规则
            // 使用cornTrigger规则
            // 触发器key
            Trigger trigger = TriggerBuilder.newTrigger().withIdentity(task.getJobName(), task.getJobGroup())
                    .startAt(DateBuilder.futureDate(1, DateBuilder.IntervalUnit.SECOND))
                    .withSchedule(CronScheduleBuilder.cronSchedule(task.getCronExpression())).startNow().build();
            trigger.getJobDataMap().put("triggerTask", JsonUtils.beanToJson(task));
            // 把作业和触发器注册到任务调度中
            Trigger.TriggerState triggerState = scheduler.getTriggerState(trigger.getKey());
            if (triggerState.equals(Trigger.TriggerState.NONE)) {
                scheduler.scheduleJob(jobDetail, trigger);
                // 启动
                if (!scheduler.isShutdown()) {
                    scheduler.start();
                    return true;
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return false;
    }

    /**
     * @description: 暂停任务
     * @return: void
     * @author: XieHeng
     * @date: 2021/4/27 5:35 下午
     */
    @Override
    public boolean pauseJob(SysTask task) {
        JobKey jobKey = JobKey.jobKey(task.getJobName(), task.getJobGroup());
        try {
            scheduler.pauseJob(jobKey);
            return true;
        } catch (SchedulerException e) {
            e.printStackTrace();
        }
        return false;
    }

    /**
     * @description: 恢复任务
     * @return: void
     * @author: XieHeng
     * @date: 2021/4/27 5:35 下午
     */
    @Override
    public boolean resumeJob(SysTask task) {
        JobKey jobKey = JobKey.jobKey(task.getJobName(), task.getJobGroup());
        try {
            scheduler.resumeJob(jobKey);
            return true;
        } catch (SchedulerException e) {
            e.printStackTrace();
        }
        return false;
    }

    /**
     * @description: 删除任务
     * @return: void
     * @author: XieHeng
     * @date: 2021/4/27 5:35 下午
     */
    @Override
    public boolean deleteJob(SysTask task) {
        JobKey jobKey = JobKey.jobKey(task.getJobName(), task.getJobGroup());
        try {
            scheduler.deleteJob(jobKey);
            return true;
        } catch (SchedulerException e) {
            e.printStackTrace();
        }

        return false;
    }

    /**
     * @description: 立马执行任务
     * @return: void
     * @author: XieHeng
     * @date: 2021/4/27 5:35 下午
     */
    @Override
    public boolean runJobNow(SysTask task) {
        JobKey jobKey = JobKey.jobKey(task.getJobName(), task.getJobGroup());
        try {
            scheduler.triggerJob(jobKey);
            return true;
        } catch (SchedulerException e) {
            e.printStackTrace();
        }
        return false;
    }

    /**
     * @description: 更新cron表达式
     * @return: void
     * @author: XieHeng
     * @date: 2021/4/27 5:36 下午
     */
    @Override
    public boolean updateJobCron(SysTask task) {
        TriggerKey triggerKey = TriggerKey.triggerKey(task.getJobName(), task.getJobGroup());
        try {
            CronTrigger trigger = (CronTrigger) scheduler.getTrigger(triggerKey);
            CronScheduleBuilder cronScheduleBuilder = CronScheduleBuilder.cronSchedule(task.getCronExpression());
            trigger = trigger.getTriggerBuilder().withIdentity(triggerKey).withSchedule(cronScheduleBuilder).build();
            scheduler.rescheduleJob(triggerKey, trigger);
            return true;
        } catch (SchedulerException e) {
            e.printStackTrace();
        }
        return false;
    }

    public static void main(String[] args) {
        URL url = Thread.currentThread().getContextClassLoader().getResource("TestJob");
        System.out.println(url.toString());
    }
}
